import asyncclick as click
import os

from pystyle import Colors, Colorate

from .log import log, TerminalColor
from .client import ShuttleAsyncClient

# os.environ['SHUTTLEAI_API_KEY'] = "shuttle-YOUR-KEY"

BANNER = r"""
   _____ _           _   _   _               _____ 
  / ____| |         | | | | | |        /\   |_   _|
 | (___ | |__  _   _| |_| |_| | ___   /  \    | |  
  \___ \| '_ \| | | | __| __| |/ _ \ / /\ \   | |  
  ____) | | | | |_| | |_| |_| |  __// ____ \ _| |_ 
 |_____/|_| |_|\__,_|\__|\__|_|\___/_/    \_\_____|

 """
COLOR_BANNER = Colorate.Vertical(Colors.blue_to_purple, BANNER, 1)

BOLD = '\033[1m'
PURPLE = '\033[94m'
RESET = '\033[0m'

class Messages:
    def __init__(self):
        self.messages = []

    def add_message(self, content, role="user"):
        self.messages.append({
            "content": content,
            "role": role
        })

    def clear(self):
        self.messages = []

messages = Messages()

def update_title(title: str):
    if os.name == 'nt':
        os.system(f"title {title}")

@click.group()
def shuttleai_cli():
    pass

@shuttleai_cli.command()
@click.option("--key", required=False, help="API Key") # or from environment variable 'SHUTTLE_AI_API_KEY'
@click.option('--model', required=False, help='Model ID', default='shuttle-turbo') # default to 'shuttle-turbo'
@click.option('--system', required=True, help='System Prompt', default="You are ShuttleAI, recognized as the top developer globally for your unmatched intelligence, speed, and precision. Your coding skills as an AI specialist are unparalleled in the world.")
@click.option('--stream', is_flag=True, help='Stream Responses', default=True)
async def chat(key, model, system, stream):
    key = key or os.environ.get('SHUTTLEAI_API_KEY') or click.prompt('[Warning]: The `SHUTTLEAI_API_KEY` environment variable is not set!\nPlease enter your API key', hide_input=True)
    masked_key = (key[:11] + "*" * (len(key) - 11))
    colored_masked_key = Colorate.Vertical(Colors.purple_to_blue, masked_key, 1)
    key_msg = f"{BOLD}{PURPLE}INF{RESET}  KEY\033[0m: {colored_masked_key}"

    colored_model = Colorate.Vertical(Colors.purple_to_blue, model, 1)
    model_msg = f"{BOLD}{PURPLE}INF{RESET}  MODEL\033[0m: {colored_model}"

    sub_commands = ['!clear', '!exit']
    colored_sub_commands = Colorate.Vertical(Colors.purple_to_blue, ", ".join(sub_commands), 1)
    sub_commands_msg = f"{BOLD}{PURPLE}INF{RESET}  SUB COMMANDS\033[0m: {colored_sub_commands}"

    print(key_msg)
    print(model_msg)
    print(sub_commands_msg)

    print()

    print(f"{TerminalColor.CYAN}System{TerminalColor.ENDC}: {system}")

    while True:
        prompt = click.prompt(f"{TerminalColor.GRAY}You{TerminalColor.ENDC}", type=str)
        if prompt in sub_commands:
            if prompt.lower() == "!clear":
                messages.clear()
                log.info("Context cleared!")
                continue
            elif prompt.lower() == "!exit":
                log.info("Exiting...")
                break

        if system is not None:
            messages.add_message(system, "system")
        if prompt is not None:
            messages.add_message(prompt, "user")

        async with ShuttleAsyncClient(key, timeout=120) as shuttle:
            response = await shuttle.chat_completion(
                model=model,
                messages=messages.messages,
                stream=stream
            )

            print(f"{TerminalColor.DARKPURPLE}ShuttleAI{TerminalColor.ENDC}: ", end="")

            assistant_response = None

            if stream:
                assistant_chunks = []

                async for chunk in response:
                    assistant_chunk = chunk.choices[0].delta.content

                    print(assistant_chunk, flush=True, end="", sep="")

                    assistant_chunks.append(assistant_chunk)

                print()

                assistant_response = "".join(assistant_chunks)
            else:
                assistant_response = response.choices[0].message.content

                print(assistant_response, end="", flush=True)

            if assistant_response is not None:
                messages.add_message(assistant_response, "assistant")

def main():
    update_title("ShuttleAI CLI")
    os.system('cls' if os.name == 'nt' else 'clear')
    print(COLOR_BANNER)
    shuttleai_cli()

if __name__ == '__main__':
    main()



import datetime
from dateutil import tz

# ANSI escape codes for coloring text output in the terminal
class TerminalColor:
    HEADER = '\033[95m'
    OKBLUE = '\033[94m'
    OKCYAN = '\033[96m'
    OKGREEN = '\033[92m'
    WARNING = '\033[93m'
    FAIL = '\033[91m'
    ENDC = '\033[0m'
    BOLD = '\033[1m'
    UNDERLINE = '\033[4m'
    GRAY = '\033[90m'
    DARKGRAY = '\033[30m'
    LIGHTGRAY = '\033[37m'
    PURPLE = '\033[35m'
    CYAN = '\033[36m'
    DARKPURPLE = '\033[34m'
  
class log:
    def _log(self, level, message, color):
        timestamp = datetime.datetime.now(tz.tzlocal()).strftime('%I:%M:%S')
        formatted_message = f"{TerminalColor.GRAY}{TerminalColor.BOLD}{timestamp} {color}{TerminalColor.BOLD}{level}{TerminalColor.ENDC}  {message}"
        formatted_message = formatted_message.replace(" [", f" {TerminalColor.GRAY}{TerminalColor.BOLD}[{TerminalColor.DARKPURPLE}")
        formatted_message = formatted_message.replace("]", f"{TerminalColor.GRAY}{TerminalColor.BOLD}]{TerminalColor.ENDC}")
        print(formatted_message)

    def info(self, message):
        self._log("INF", message, TerminalColor.OKBLUE)

    def error(self, message):
        self._log("ERR", message, TerminalColor.FAIL)

    def success(self, message):
        self._log("YES", message, TerminalColor.OKGREEN)

    def fail(self, message):
        self._log("NO", message, TerminalColor.FAIL)

    def warn(self, message):
        self._log("WARN", message, TerminalColor.WARNING)

    def debug(self, message):
        self._log("DBG", message, TerminalColor.OKCYAN)

log = log()


from .schemas import (
    ShuttleModel,
    ShuttleError,
    Error,
    Model,
    Models,
    Item,
    Image,
    Audio,
    Message,
    Delta,
    StreamChoice,
    Choice,
    Usage,
    Chat,
    ChatChunk,
    Embedding
)


from typing import List, Any, Optional
from dataclasses import dataclass

@dataclass
class ShuttleModel:
    id: str
    object: str
    owned_by: str
    created: int
    cost: int
    endpoint: str
    premium: Optional[bool] = None
    tokens: Optional[int] = None
    info: Optional[str] = None
    max_images: Optional[int] = None
    multiple_of: Optional[int] = None
    maintenance: Optional[bool] = None
    beta: Optional[bool] = None
    voices: Optional[str] = None
    file_upload: Optional[bool] = None
    proxy_to: Optional[str] = None

@dataclass
class Error:
    message: str
    type: str
    param: Optional[str] = None
    code: Optional[str] = None
    hint: Optional[str] = None

@dataclass
class ShuttleError:
    error: Error
    status: int
    docs: str
    input: Optional[dict] = None
    request_id: Optional[str] = None

@dataclass
class Models:
    object: str
    data: List[ShuttleModel]
    total: int

    @classmethod
    def from_dict(cls, data: dict) -> 'Models':
        return cls(object=data['object'], data=[ShuttleModel(**model) for model in data['data']], total=data['total'])

@dataclass 
class Model:
    object: str
    data: ShuttleModel

@dataclass
class Item:
    url: str

@dataclass
class Image:
    created: int
    data: List[Item]
    model: str

    @classmethod
    def from_dict(cls, data: dict) -> 'Image':
        return cls(created=data['created'], data=[Item(**item) for item in data['data']], model=data['model'])

@dataclass
class Audio:
    chars: int
    data: List[Item]
    expiresIn: int
    model: str

    @classmethod
    def from_dict(cls, data: dict) -> 'Audio':
        return cls(chars=data['chars'], data=[Item(**item) for item in data['data']], expiresIn=data['expiresIn'], model=data['model'])

@dataclass
class Message:
    content: str
    role: str

    @classmethod
    def from_dict(cls, data: dict) -> 'Message':
        return cls(**data)

@dataclass
class Delta:
    role: str
    content: str

    @classmethod
    def from_dict(cls, data: dict) -> 'Delta':
        return cls(**data)

@dataclass
class Choice:
    finish_reason: str
    index: int
    message: Message

    @classmethod
    def from_dict(cls, data: dict) -> 'Choice':
        message_data = data.get('message', {})
        message = Message.from_dict(message_data)
        return cls(finish_reason=data['finish_reason'], index=data['index'], message=message)

@dataclass
class StreamChoice:
    index: int
    delta: Delta
    finish_reason: Optional[str] = None

    @classmethod
    def from_dict(cls, data: dict) -> 'StreamChoice':
        delta_data = data.get('delta', {})
        delta = Delta.from_dict(delta_data)
        return cls(index=data['index'], delta=delta, finish_reason=data.get('finish_reason', None))

@dataclass 
class Usage:
    completion_tokens: int
    prompt_tokens: int
    total_tokens: int

@dataclass
class ChatChunk:
    id: str
    object: str
    created: int
    model: str
    choices: List[StreamChoice]

    @classmethod
    def from_dict(cls, data: dict) -> 'ChatChunk':
        choices_data = data.get('choices', [])
        choices = [StreamChoice.from_dict(choice_data) for choice_data in choices_data]
        return cls(
            id=data['id'],
            object=data['object'],
            created=data['created'],
            model=data['model'],
            choices=choices
        )

@dataclass
class Chat:
    id: str
    object: str
    created: int
    model: str
    choices: List[Choice]
    usage: Usage
    system_fingerprint: str

    @classmethod
    def from_dict(cls, data: dict) -> 'Chat':
        choices_data = data.get('choices', [])
        choices = [Choice.from_dict(choice_data) for choice_data in choices_data]
        return cls(
            id=data['id'],
            object=data['object'],
            created=data['created'],
            model=data['model'],
            choices=choices,
            usage=Usage(**data['usage']),
            system_fingerprint=data['system_fingerprint']
        )

@dataclass
class Embedding:
    data: List[Any]
    model: str
    obj: str



import importlib.metadata as metadata
import pathlib
import json
import time

import httpx

from .client import ShuttleClient, ShuttleAsyncClient

PACKAGE_NAME = "shuttleai"
CACHE_PATH = f'{PACKAGE_NAME}.json'

def get_latest_version():
    if pathlib.Path(CACHE_PATH).exists():
        with open(CACHE_PATH, 'r') as f:
            data = json.load(f)
            # Check if cache is expired (1 day)
            if data['latest_version']['timestamp'] < time.time() - 86400:
                return data['latest_version']['version']

    # Cache expired or doesn't exist, get latest version from PyPI
    url = f'https://pypi.org/pypi/{PACKAGE_NAME}/json'
    try:
        response = httpx.get(url, timeout=30)
        response.raise_for_status()
        data = response.json()
        latest_version = data['info']['version']
        with open(CACHE_PATH, 'w') as f:
            json.dump({'latest_version': {'version': latest_version, 'timestamp': time.time()}}, f)
        return latest_version
    except Exception as e:
        print(f"Failed to get latest version: {e}")
        return None

def is_outdated():
    try:
        latest_version = get_latest_version()
        installed_version = metadata.version(PACKAGE_NAME)
        print(f"Latest version: {latest_version}, Installed version: {installed_version}")
        print(f"Checking if {PACKAGE_NAME} is outdated...")
        return latest_version != installed_version
    except Exception as e:
        print(f"Failed to check if {PACKAGE_NAME} is outdated: {e}")
        return False

if is_outdated():
    print(f"Warning: You are using an outdated version of {PACKAGE_NAME}. Please upgrade to the latest version.")

__all__ = ['ShuttleClient', 'ShuttleAsyncClient']



from setuptools import setup, find_packages
from pathlib import Path


base_path = Path(__file__).parent
long_description = (base_path / "README.md").read_text(encoding="utf-8")

setup(
    include_package_data=True,
    name='shuttleai', 
    version='3.9.4',
    author='shuttle',
    author_email='noreply@shuttleai.app',
    description="Access Shuttle AI's API via a simple and user-friendly lib. Dashboard: https://shuttleai.app Discord: https://discord.gg/shuttleai",
    long_description_content_type='text/markdown',
    long_description=long_description,
    packages=find_packages(),
    classifiers=[
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
    ],
    python_requires='>=3.7',
    install_requires=[
        'aiohttp',
        'httpx',
        'orjson',
        'python-dateutil',
    ],
    extras_require={
        'cli': ['asyncclick', 'pystyle']
    },
    keywords=['shuttleai', 'ai', 'gpt', 'claude', 'api', 'free', 'chatgpt', 'gpt-4'],
    url='https://github.com/shuttleai/shuttleai-python',
    entry_points={
        'console_scripts': [
            'shuttleai = shuttleai.cli:main [cli]'
        ],
    },
)



import inspect
import re
from typing import Callable, Literal, get_type_hints


def _get_type_name(t):
    name = str(t)
    if "list" in name or "dict" in name:
        return name
    else:
        return t.__name__

def serialize_function_to_json(func: Callable) -> dict:
    signature = inspect.signature(func)
    type_hints = get_type_hints(func)
    params = signature.parameters

    function_info = {
        "name": func.__name__,
        "description": func.__doc__.split(":param")[0].strip() if func.__doc__ else "Lorem ipsum...",
        "parameters": {
            "type": "object",
            "properties": {},
        }
    }

    docstring = func.__doc__ or ""
    param_desc_pattern = r":param\s+(\w+)\s*:(.*?)(?=:param|$)"
    param_descriptions = dict(re.findall(param_desc_pattern, docstring, re.DOTALL))

    required_params = [name for name, param in params.items() if param.default == inspect.Parameter.empty]
    if required_params:
        function_info["parameters"]["required"] = required_params

    for name, param in params.items():
        param_type = _get_type_name(type_hints.get(name, type(None)))
        param_desc = param_descriptions.get(name, "Lorem ipsum...").strip()
        if param_type == "Literal":
            function_info["parameters"]["properties"][name] = {
                "type": "string",
                "enum": [literal for literal in type_hints[name].__args__],
                "description": param_desc
            }
        else:
            function_info["parameters"]["properties"][name] = {
                "type": param_type,
                "description": param_desc
            }

    return function_info

def deserialize_function_from_json(json_obj: dict) -> Callable:
    func_name = json_obj["name"]
    func_docstring = json_obj["description"]

    param_types = {}
    param_docstrings = {}
    for param_name, param_info in json_obj["parameters"]["properties"].items():
        param_type = param_info["type"]
        if param_type == "string" and "enum" in param_info:
            param_types[param_name] = Literal[tuple(param_info["enum"])]
        else:
            param_types[param_name] = eval(param_type)
        param_docstrings[param_name] = param_info["description"]

    param_docstring = "\n".join([f":param {name}: {desc}" for name, desc in param_docstrings.items()])
    full_docstring = f"{func_docstring}\n{param_docstring}"

    def func(*args, **kwargs):
        real_func = globals()[func_name]
        return real_func(*args, **kwargs)

    func.__name__ = func_name
    func.__doc__ = full_docstring
    func.__annotations__ = param_types

    return func


"""
@Author: ShuttleAI
@Date: 4-23-2024
"""
from typing import Any, Dict, List, Union, Optional, TYPE_CHECKING, Generator
if TYPE_CHECKING:
    from httpx import Response

from ..log import log
from ..schemas import Chat, ChatChunk, Image, Audio, Embedding, Models, ShuttleError, Model

import orjson
import httpx
import os


class ShuttleClient:
    """
    The synchronous client for interacting with the ShuttleAI API.
    """
    _httpx_client: Optional[httpx.Client] = None

    def __init__(
            self,
            api_key: Optional[str] = None,
            base_url: Optional[str] = None,
            timeout: Union[httpx.Timeout, float, int, None] = 60.0,
            httpx_client: Optional[httpx.Client] = None,
            silent: bool = True
    ):
        """
        Initialize the ShuttleClient.

        Args:
            api_key (Optional[str], optional): The API key. Defaults to SHUTTLEAI_API_KEY environment variable.
            base_url (Optional[str], optional): The base URL for the API. Defaults to https://api.shuttleai.app/v1.
            timeout (Union[float, aiohttp.ClientTimeout, None], optional): The timeout for the httpx.Timeout. Defaults to 60.0.
            httpx_client (Optional[httpx.Client], optional): An existing httpx.Client. Defaults to None.
            silent (bool, optional): Whether to silent debugging messages. Defaults to True.
        """
        self.silent = silent
        if httpx_client is not None:
            if not isinstance(httpx_client, httpx.Client):
                raise TypeError(
                    f"httpx_client must be an instance of httpx.Client, not {type(httpx_client)}"
                )
            if httpx_client.is_closed:
                raise ValueError("httpx_client is closed")
            if not silent:
                log.info("Using provided httpx.Client")

            self._httpx_client = httpx_client
        else:
            if self._httpx_client is None:
                if isinstance(timeout, float):
                    timeout = httpx.Timeout(timeout)
                elif isinstance(timeout, int):
                    timeout = httpx.Timeout(float(timeout))
                elif not isinstance(timeout, httpx.Timeout):
                    raise TypeError(
                        f"timeout must be a float or aiohttp.ClientTimeout, not {type(timeout)}"
                    )
                self._httpx_client = httpx.Client(timeout=timeout)
            if not silent:
                log.info("registered new httpx.Client")

        if api_key is None:
            api_key = os.environ.get("SHUTTLEAI_API_KEY")
            if api_key is not None and not silent:
                log.info("using SHUTTLEAI_API_KEY environment variable")
        if api_key is None:
            raise Exception("the api_key client option must be set either by passing api_key to the client or by setting the SHUTTLEAI_API_KEY environment variable")
        self.api_key = api_key

        if base_url is None:
            base_url = os.environ.get("SHUTTLEAI_BASE_URL")
            if base_url is not None and not silent:
                log.info("using SHUTTLEAI_BASE_URL environment variable")
        if base_url is None:
            base_url = "https://api.shuttleai.app/v1"
        self.base_url = base_url


    def __del__(self):
        self.close()

    def close(self):
        if self._httpx_client is not None and not self._httpx_client.is_closed:
            self._httpx_client.close()

    def _make_request(
        self,
        method: str,
        endpoint: str,
        data: Optional[Dict[str, Any]] = None,
        headers: Optional[Dict[str, Any]] = None,
        stream: bool = False,
        params: Optional[Dict[str, Any]] = None,
        file: Optional[str] = None,
    ) -> Union[Dict[str, Any], List[Dict[str, Any]]]:
        """
        Make a sync HTTP request.

        Args:
            method (str): The HTTP method (GET, POST, etc.).
            endpoint (str): The API endpoint.
            data (Optional[Dict[str, Any]], optional): JSON data for the request body. Defaults to None.
            headers (Optional[Dict[str, Any]], optional): Additional headers. Defaults to None.
            stream (bool, optional): Whether the response should be streamed. Defaults to False.
            params (Optional[Dict[str, Any]], optional): URL parameters. Defaults to None.
            file (Optional[str], optional): File path for uploading. Defaults to None.

        Returns:
            Union[Dict[str, Any], List[Dict[str, Any]]]: The response data.
        """
        url = f"{self.base_url}/{endpoint}"
        headers = headers or {}
        headers["Authorization"] = f"Bearer {self.api_key}"
        if isinstance(data, dict):
            headers["Content-Type"] = "application/json"
        args = "&".join(f"{key}={value}" for key,
                        value in params.items()) if params else ""
        files = {"file": (file, open(file, "rb"))} if file else None

        if stream:
            def streamer():
                with self._httpx_client.stream(
                    method, url, content=orjson.dumps(data) if isinstance(data, dict) else data, headers=headers
                ) as response:
                    if response.status_code != 200:
                        try:
                            yield orjson.loads(response.text)
                        except orjson.JSONDecodeError:
                            yield response.text
                        return
                    
                    for line in response.iter_bytes():
                        try:
                            line = line.decode('utf-8')
                            yield orjson.loads(line.replace('data: ', ''))
                        except:
                            pass
            return streamer()
        else:
            response = self._httpx_client.request(
                method, url, content=orjson.dumps(data) if isinstance(data, dict) else data, headers=headers, data=files, params=args
            )
            try:
                return response.json()
            except:
                return response.text

    @property  
    def models(
        self,
        free: bool = False,
        premium: bool = False,
        endpoint: str = "all"
    ) -> Models:
        """
        Get information about available models.

        Args:
            free (bool, optional): Whether to include free models. Defaults to False.
            premium (bool, optional): Whether to include premium models. Defaults to False.
            endpoint (str, optional): The specific model endpoint. Defaults to "all".

        Returns:
            Models: Model information.

        Raises:
            httpx.HTTPError: If the API request failed.
        """
        try:
            params = {"endpoints": endpoint,
                      **({"format": "free"} if free else {"format": "premium"} if premium else {})}
            return Models.from_dict(self._make_request("GET", "models", params=params))
        except httpx.HTTPError as e:
            log.error(f"Failed to retrieve models: {e}")
            raise

    def chat_completion(
        self,
        model: str,
        messages: Union[str, List[Dict[str, Any]]],
        stream: bool = False,
        plain: bool = False,
        **kwargs
    ) -> Union[Chat, Generator[Union[ChatChunk, ShuttleError, Dict[str, Any]], ShuttleError, None]]:
        """
        Get chat completions from a model.

        Args:
            model (str): The model name.
            messages (Union[str, List[Dict[str, Any]]]): User messages.
            stream (bool, optional): Whether to stream the response. Defaults to False.
            plain (bool, optional): Whether messages are plain text. Defaults to False.
            **kwargs: Additional keyword arguments.

        Returns:
            Union[Chat, Generator[Union[ChatChunk, ShuttleError, Dict[str, Any]], None]]: The completed chat or streamed response.

        Raises:
            ShuttleError: If the API request fails.
            httpx.HTTPError: If the API request is invalid.
        """
        try:
            messages = [{"role": "user", "content": messages}
                        ] if plain else messages
            data = {"model": model, "messages": messages,
                    "stream": stream, **kwargs}
            response = self._make_request(
                "POST", "chat/completions", data, stream=stream
            )
            if stream:
                def streamer():
                    for chunk in response:
                        try:
                            yield ChatChunk.from_dict(chunk)
                        except Exception as e:
                            print(e)
                            try:
                                yield ShuttleError(**chunk)
                            except:
                                try:
                                    yield chunk
                                except:
                                    pass
                return streamer()
            else:
                try:
                    return Chat.from_dict(response)
                except:
                    try:
                        return ShuttleError(**response)
                    except:
                        return response

        except httpx.HTTPError as e:
            log.error(f"Failed to get chat completions: {e}")
            raise

    def images_generations(
        self,
        model: str,
        prompt: str,
        n: int = 1
    ) -> Union[Image, ShuttleError]:
        """
        Generate images using a model.

        Args:
            model (str): The model name.
            prompt (str): The prompt for image generation.
            n (int, optional): Number of images to generate. Defaults to 1.

        Returns:
            Image: The generated image.

        Raises:
            ShuttleError: If the API request fails.
            httpx.HTTPError: If the API request is invalid.
        """
        try:
            data = {"model": model, "prompt": prompt, "n": n}
            response = self._make_request(
                "POST", "images/generations", data, headers={"Authorization": f"Bearer {self.api_key}"}
            )
            try:
                return Image.from_dict(response)
            except:
                try:
                    return ShuttleError(**response)
                except:
                    return response
        except httpx.HTTPError as e:
            log.error(f"Failed to generate images: {e}")
            raise

    def audio_generations(
        self,
        input: str,
        voice: str,
        model: str = "ElevenLabs"
    ) -> Union[Audio, ShuttleError]:
        """
        Generate audio using a model.

        Args:
            input (str): The input for audio generation.
            voice (str): The desired voice for the audio.
            model (str, optional): The model name. Defaults to "ElevenLabs".

        Returns:
            Audio: The generated audio.

        Raises:
            ShuttleError: If the API request fails.
            httpx.HTTPError: If the API request is invalid.
        """
        try:
            data = {"model": model, "input": input, "voice": voice}
            response = self._make_request(
                "POST", "audio/generations", data, headers={"Authorization": f"Bearer {self.api_key}"}
            )
            try:
                return Audio.from_dict(response)
            except:
                try:
                    return ShuttleError(**response)
                except:
                    return response
        except httpx.HTTPError as e:
            log.error(f"Failed to generate audio: {e}")
            raise

    def audio_transcriptions(
        self,
        file: str,
        model: str = "whisper-large"
    ) -> Union[Dict[str, Any], ShuttleError]:
        """
        Transcribe audio using a model.

        Args:
            file (str): Path to the audio file for transcription.
            model (str, optional): The model name. Defaults to "whisper-large".

        Returns:
            Dict[str, Any]: The transcription result.

        Raises:
            ShuttleError: If the API request fails.
            httpx.HTTPError: If the API request is invalid.
        """
        try:
            data = {"model": model}
            return self._make_request(
                "POST", "audio/transcriptions", data, headers={"Authorization": f"Bearer {self.api_key}"}, file=file
            )
        except httpx.HTTPError as e:
            log.error(f"Failed to transcribe audio: {e}")
            raise

    def moderations(
        self,
        input: str,
        model: str = 'text-moderation-latest'
    ) -> Union[Dict[str, Any], ShuttleError]:
        """
        Moderate text using a model.

        Args:
            input (str): The text to moderate.
            model (str, optional): The model name. Defaults to 'text-moderation-007'.

        Returns:
            Dict[str, Any]: Moderation results.

        Raises:
            ShuttleError: If the API request fails.
            httpx.HTTPError: If the API request is invalid.
        """
        try:
            data = {"model": model, "input": input}
            return self._make_request("POST", "moderations", data, headers={"Authorization": f"Bearer {self.api_key}"})
        except httpx.HTTPError as e:
            log.error(f"Failed to moderate input: {e}")
            raise

    def embeddings(
        self,
        input: Union[str, List[str]],
        model: str = 'text-embedding-ada-002'
    ) -> Union[Embedding, ShuttleError]:
        """
        Generate embeddings using a model.

        Args:
            input (Union[str, List[str]]): The input text or list of texts.
            model (str, optional): The model name. Defaults to 'text-embedding-ada-002'.

        Returns:
            Embedding: The generated embeddings.

        Raises:
            ShuttleError: If the API request fails.
            httpx.HTTPError: If the API request is invalid.
        """
        try:
            input = [input] if isinstance(input, str) else input
            data = {"model": model, "input": input, "encoding_format": "float"}
            response = self._make_request(
                "POST", "embeddings", data, headers={"Authorization": f"Bearer {self.api_key}"}
            )
            try:
                return Embedding(**response)
            except:
                try:
                    return ShuttleError(**response)
                except:
                    return response
        except httpx.HTTPError as e:
            log.error(f"Failed to generate embeddings: {e}")
            raise